package se.chalmers.cs.gf.abssyn;

import SyntaxTree.*;
import SyntaxTree.Absyn.*;

/**
 *  Builds an untyped GF abstract syntax tree from a tree
 *  in the BNFC-generated abstract syntax for trees.
 */
public class GFTreeToTree implements Tr.Visitor<Tree,Integer> {

        public static Tree toTree(Tr t) {
                return t.accept(new GFTreeToTree(), 0);
        }
        
        public Tree visit(SyntaxTree.Absyn.TNode tnode, Integer arg) {
                Tree[] children = new Tree[tnode.listtr_.size()];
                int i = 0;
                for (Tr c : tnode.listtr_)
                        children[i++] = c.accept(this, arg);
                return new Fun(tnode.ident_, children);
        }

        public Tree visit(SyntaxTree.Absyn.TAtom tatom, Integer arg) {
                return new Fun(tatom.ident_);
        }

        public Tree visit(SyntaxTree.Absyn.TStr tstr, Integer arg) {
                return new StringLiteral(tstr.string_);
        }

        public Tree visit(SyntaxTree.Absyn.TInt tint, Integer arg) {
                return new IntLiteral(tint.integer_);
        }

}
