package se.chalmers.cs.gf.oaa;

import se.gu.ling.trindikit.oaa.common.*;
import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.GFException;
import se.chalmers.cs.gf.abssyn.*;
import se.chalmers.cs.gf.translate.*;
import se.chalmers.cs.gf.util.Pair;
import static se.chalmers.cs.gf.util.Pair.pair;

import java.util.*;

/**
 *  Implements the parsing solvable.
 */
public class Parse extends OAASolver {

        private static String solvableString = "parse(Grammar,Lang,Text,Tree)";

        private Translators translators;

        public Parse(Translators translators) {
                super(solvableString);
                this.translators = translators;
        }

        public boolean solve(IclTerm goal, IclList params, IclList answers) {
                String grammar;
                IclTerm langTerm;
                String text;
                IclTerm result;

                try {
                        grammar = ((IclStr)goal.getTerm(0)).toUnquotedString();
                        langTerm = goal.getTerm(1);
                        text = ((IclStr)goal.getTerm(2)).toUnquotedString();
                        result = goal.getTerm(3);
                } catch (ClassCastException e){
                        return false;
                }

                Translator translator = translators.getTranslator(grammar);
                if (translator == null) {
                        System.err.println("Unknown grammar " + grammar);
                        return false;
                }

                Collection<Pair<String,Tree>> ts = parse(translator, langTerm, text);

                System.err.println("Parser returned " + ts.size() + " results.");

                boolean success = false;
                for (Pair<String,Tree> p : ts) {
                        String lang = p.fst;
                        Tree t = p.snd;
                        IclTerm parseResult = TreeToIclTerm.treeToIclTerm(t);
                        System.err.println("Trying to unify " + result + " and " + parseResult);
                        IclTerm r = Unifier.getInstance().unify(result, parseResult);
                        if (r != null) {
                                IclTerm ans = new IclStruct("parse", new IclStr(grammar),
                                                            new IclStr(lang), new IclStr(text), r);

                                System.err.println("Adding answer " + ans);
                                answers.add(ans);
                                success = true;
                        }
                }

                System.err.println("Returning " + success);
                return success;
        }

        public static Collection<Pair<String,Tree>> parse(Translator translator, 
                                                          IclTerm langTerm, String text) {
                try {
                        if (langTerm.isStr()) {
                                String lang = ((IclStr)langTerm).toUnquotedString();
                                System.err.println("Parsing '" + text + "' as " + lang);
                                List<Pair<String,Tree>> ts = new LinkedList<Pair<String,Tree>>();
                                for (Tree t : translator.parse(lang, text))
                                        ts.add(pair(lang,t));
                                return ts;
                        } else if (langTerm.isVar()) {
                                System.err.println("Parsing '" + text + "' as any language");
                                return translator.parseWithAll(text);
                        } else {
                                System.err.println("Bad language " + langTerm);
                                return new LinkedList<Pair<String,Tree>>();
                        }
                } catch (GFException ex) {
                        ex.printStackTrace();
                        return new LinkedList<Pair<String,Tree>>();
                }
        }

}
