package se.chalmers.cs.gf.parse.chart;

import java.util.*;

public abstract class Rule {

        /**
         *  Must be unique with respect to all other rules in the grammar.
         */
        private String name;

        private Category cat;


        public Rule(String name, Category cat) {
                this.name = name;
                this.cat = cat;
        }

        /**
         *  Get the unique name of the rule.
         */
        public String getName() {
                return name;
        }

        /**
         *  Get the left hand side of the rule.
         */
        public Category getCategory() {
                return cat;
        }

        /**
         *  Get the function name use to construct an abstract syntax
         *  tree from this rule.
         */
        public abstract String getFun();

        /**
         *  Get the number of terminals or non-terminal on the RHS.
         */ 
        public abstract int getSize();

        /**
         *  Checks if the right hand side of this rule consists of a 
         *  single terminal.
         */
        public abstract boolean isTerminal();
        
}
